;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-
;; See the file LICENCE for licence information.

;; Miscellaneous utilities used throughout the package.
(in-package :cl-store)
(declaim (optimize (speed 3) (safety 0) (debug 0)))



(defmacro aif (test then &optional else)
  `(let ((it ,test))
    (if it ,then ,else)))

(defmacro with-gensyms (names &body body)
  `(let ,(mapcar #'(lambda (x) `(,x (gensym))) names)
    ,@body))

(defun mappend (fn &rest lsts)
  (apply #'append (apply #'mapcar fn lsts)))


(defun get-slot-details (slot-definition)
  "Return a list of slot details which can be
   used as an argument to ensure-class"
  (list :name (slot-definition-name slot-definition)
        :allocation (slot-definition-allocation slot-definition)
        :initargs (slot-definition-initargs slot-definition)
        ;; :initform. dont use initform until we can
        ;; serialize functions
        :readers (slot-definition-readers slot-definition)
        :type (slot-definition-type slot-definition)
        :writers (slot-definition-writers slot-definition)))

(defun get-class-details (x)
  "Return a list of class details which can be 
   used as arguments to ensure-class"
  (list (class-name x)
        ;; can't use this value either (see get-slot-details)
        ;;(class-direct-default-initargs x)
        (mapcar #'get-slot-details (class-direct-slots x))
        (mapcar #'class-name
                (class-direct-superclasses x))
        (type-of x)))


(defmacro awhen (test &body body)
  `(aif ,test
    (progn ,@body)))


;; because clisp doesn't have the class single-float or double-float.
(defun float-type (float)
  (typecase float
    (single-float 0)
    (double-float 1)
    (t 0)))

(defun get-float-type (num)
  (case num
    (0 1.0)
    (1 1.0d0)))


(defun store-32-byte (obj stream)
  "Write OBJ down STREAM as a 32 byte integer."
  (write-byte (ldb (byte 8 0) obj) stream)
  (write-byte (ldb (byte 8 8) obj) stream)
  (write-byte (ldb (byte 8 16) obj) stream)
  (write-byte (+ 0 (ldb (byte 8 24) obj)) stream))


(defun read-32-byte (buf &optional (signed t))
  "Read a signed or unsigned byte off STREAM."
  (let ((byte1 (read-byte buf))
        (byte2 (read-byte buf))
        (byte3 (read-byte buf))
        (byte4 (read-byte buf)))
    (declare (type (mod 256) byte1 byte2 byte3 byte4))
    (let ((ret (+ byte1 (* 256 (+ byte2 (* 256 (+ byte3 (* 256 byte4))))))))
      (if (and signed (> byte1 127))
          (logior (ash -1 32) ret)
          ret))))


(defun store-string-code (string stream)
  "Write length of STRING then STRING into stream"
  (declare (type simple-string string))
  (format stream "~S" string))

(defun retrieve-string-code (stream)
  "Retrieve a String written by store-string-code from STREAM"
  (read stream))


;; EOF